#!/usr/bin/awk -f

# parser.awk - parsing script for awkiawki
# 
# Copyright (c) 2002 Oliver Tonnhofer <olt@bogosoft.com>
# Copyright (c) 2003-2005 Paul H. Liu <paul@thev.net>
#
# See the file `COPYING' for copyright notice.

BEGIN {
    index_anchor = 0
    list["ol"] = 0
    list["ul"] = 0
    scriptname = ENVIRON["SCRIPT_NAME"]
    if (match(scriptname, /[^\/]+$/))
        pathname = substr(scriptname, 1, RSTART)
    FS = "[ ]"
    wikiexp1 = "[A-Z]+[a-z]+[A-Z][A-Za-z]*|[A-Z][A-Z]+[a-z][A-Za-z]*"
    wikiexp2 = "_(\\w|[\\\\.:_-])+_"
    wikiexp = wikiexp1 "|" wikiexp2
        
    cmd = special_list " " datadir
    while ((cmd | getline ls_out) >0)
    {
        if (ls_out ~ ("^(" wikiexp ")(\\t([0-9a-fA-F]+))?$")) {
            if (match(ls_out, /\t/) > 1) {        
                page = substr(ls_out, 1, RSTART - 1)
                rev = substr(ls_out, RSTART + 1)
            }
            else {
                page = ls_out
                rev = 1
            }
            if (page !~ /,v/) {
                pages[page] = rev
            }
        }
    }
    close(cmd)
}

#
# handle embeded code fragment.
# we use enscript to beautify codes in HTML
#
/^%ENDCODE%$/ {
    print "<div class=\"fragment\">"
    cmd = "enscript --color -E'" embeded_language "' --language=html -o -|" \
          "awk '/^<HR>/ { p = 0; } /^<PRE>/ { p = 1; } (p == 1) {print;}'"
    print embeded_code | cmd
    close(cmd)
    print "</div>"
    embeded = "false"
    embeded_code = ""
    embeded_language = ""
    next
}

/^%CODE{\w+}%$/ {    
    sub("%CODE{", "")
    sub("}%", "")
    embeded = "true"
    embeded_code = ""
    embeded_language = $0
    close_tags()
    next
}

embeded == "true" {
    if (embeded_code == "") embeded_code = $0
    else embeded_code = embeded_code "\n" $0
    next
}

#
# generate indices by gathering all headers in the same page
#
/^%INDEX%$/ {
    if (revision) {
        system("co -q -p'"revision"' " filename " | " \
               special_parser " -v special_summary=true")
    }
    else {
        system(special_parser " -v special_summary=true " ARGV[1])
    }
    next
}

#
# produce a link to comment
# 
/^%COMMENT(:\w+)?%$/ {
    close_tags();

    com = "<p/><span class=\"commenttitle\">Comments</span>" \
          "<font size=-1><a href=\"" scriptname \
          "/" mypage "?post=true\">post comment</a><tt>&nbsp;&nbsp;&nbsp;</tt>"

    sub("%COMMENT", "")
    sub("%", "")
    order = $0

    cmd = special_list " " datadir "COMMENT/" mypage "/"
    n = 1

    while ((cmd | getline ls_out) >0)
    {
        if (match(ls_out, /\t/) > 1) {
            page = substr(ls_out, 1, RSTART - 1)
            edittime = substr(ls_out, RSTART + 1)
            if (match(page, /[.]/) > 1) {
                filelist[n] = page "\t" \
                        substr(page, 1, RSTART - 1) "\t" edittime "\t" \
                        substr(page, RSTART + 1)
                n = n + 1
            }
        }
    }    
    close(cmd)
    n = n - 1

    if (pagesize && pageno != "all")
    {
        if (pageno == "") pageno = 1
        total = n / pagesize
        startno = pagesize * (pageno - 1) + 1
        endno = pagesize * pageno
        if (endno > n) endno = n
    } else {
        total = 0
        startno = 1
        endno = n
    }

    if (total > 1)
    {
        com = com "[ "        
        for (i = 1; i < total + 1; i++)
        {
            if (i == pageno) 
            {
                com = com i " "
            } else {
                com = com "<a href=\"" scriptname "/" mypage "?postlist=" i \
                      "\">" i " </a>"
            }
        }
        com = com "<a href=\"" scriptname "/" mypage "?postlist=all\">" \
                  "all</a> ]"
    }
    com = com "</font>"
    print com "<div id=\"commentmeta\">"

    c = 1
    if (order == ":NEWEST") {
        for (i = n; i >= 1; i--)
        {
            if (c >= startno && c <= endno)
            {
                split(filelist[i], m, "\t")
                parse_comment(m[1], m[2], m[3], m[4])
            }
            c++
        }
    }
    else {
        for (i = 1; i <= n; i++)
        {
            if (c >= startno && c <= endno)
            {
                split(filelist[i], m, "\t")
                parse_comment(m[1], m[2], m[3], m[4])
            }
            c++
        }
    }
    print "</div>" com

    next
}

#
# register blanklines
# this is to help compressing multiple blanklines into one
#
/^$/ { blankline = 1; next; }

#
# quote HTML entities for <, > and &
# permitted tags are <font> </font> <a> </a> <blockquote> </blockquote>
#
/[&<>]/ && (! /<\/?(font|blockquote|a)(>| [^>]*>)/) ||
/<\/?(font|blockquote|a)(>| [^>]*>)''''''/ {
    gsub(/&/, "\\&amp;");        
    gsub(/</, "\\&lt;"); 
    gsub(/>/, "\\&gt;");
}

#
# generate links for wikiexp, direct http tag, attachment link,
# and [[ url ][ name ]]
#
(wikiexp) ||
/(https?|ftp|gopher|mailto|news):/ ||
/%%(\w|[ \/.:-])+%%/ ||
/\[\[[^\]]+\]\[[^\]]+\]\]/ {
    tmpline = ""
    for(i = 1; i <= NF; i++) {
        field = $i 

        # generate link for local download
        if (field ~ /%%(\w|[ \/.:-])+%%/) {
            match(field, /%%(\w|[ \/.:-])+%%/)
            attachment = substr(field, RSTART + 2, RLENGTH - 4)
            realfile = attachment
            if (realfile !~ /\//) {
                realfile = mypage "/" realfile
            }
            if (attachment ~ /.*\.(jpg|jpeg|gif|png)$/) {
                sub("%%" attachment "%%", \
                    "<img border=0 alt=\"" attachment "\" src=\"" \
                    "UPLOAD/"realfile"\">", field)
            }
            else {
                sub("%%" attachment "%%", \
                    "<a href=\"UPLOAD/"realfile"\">" attachment \
                    "</a>", field)
            }
        }

        # generate [[ .. ][ .. ]] style links
        else if (field ~ /\[\[[^\]]+\]\[[^\]]+\]\]/) {
            match(field, /\[\[[^\]]+\]\[/)
            link = substr(field, RSTART + 2, RLENGTH - 4)
            gsub("&", "\\\\&", link)
            match(field, /\]\[[^\]]+\]\]/)
            name = substr(field, RSTART + 2, RLENGTH - 4)
            gsub("_", " ", name)
            sub(/\[\[[^\]]+\]\[[^\]]+\]\]/, \
                "<a href=\"" link "\">" name "</a>", field)
        }

        # generate HTML img tag for .jpg,.jpeg,.gif,png URLs
        else if (field ~ /https?:\/\/[^\t]*\.(jpg|jpeg|gif|png)/ \
                 && field !~ /''''''/) {
            sub(/https?:\/\/[^\t]*\.(jpg|jpeg|gif|png)/, \
                "<img src=\"&\">", field)        
        }

        # links for mailto, news and http, ftp and gopher URLs
        else if (field ~ /((https?|ftp|gopher):\/\/|(mailto|news):)[^\t]*/ && 
                 field !~ /["]((https?|ftp|gopher):\/\/|(mailto|news):)[^\t]*["]/) {
            sub(/((https?|ftp|gopher):\/\/|(mailto|news):)[^\t]*[^.,?;:'")\t]/,\
                "<a href=\"&\">&</a>", field)
             # remove mailto: in link description
            sub(/>mailto:/, ">",field)
        }

        # links for awkipages
        else if (field ~ (wikiexp2)  && field !~ /''''''/) {
            match(field, wikiexp2)
            tmp_pagename = substr(field, RSTART, RLENGTH)
            visname = substr(field, RSTART + 1, RLENGTH - 2)
            gsub(/_/, " ", visname)
                    
            if (pages[tmp_pagename])
                sub(wikiexp2, "<a href=\""scriptname"/&?stamp=" \
                    pages[tmp_pagename] "\">"visname"</a>", field)
            else
                sub(wikiexp2, visname "<a href=\"" \
                    scriptname "/&\">?</a>", field)
        }
        else if (field ~ ("(^|[[,.?;:'\"\\(\\t])(" wikiexp1 ")") && 
                 field !~ /''''''/) {
            match(field, wikiexp1)
            tmp_pagename = substr(field, RSTART, RLENGTH)
            if (pages[tmp_pagename])
                sub(wikiexp1, "<a href=\""scriptname"/&?stamp=" \
                    pages[tmp_pagename] "\">&</a>", field)
            else
                sub(wikiexp1, "&<a href=\""scriptname"/&\">?</a>", field)
        }
        tmpline = tmpline field OFS
    }
    # return tmpline to $0 and remove last OFS (whitespace)
    $0 = substr(tmpline, 1, length(tmpline)-1)
}

# remove six single quotes (Wiki''''''Links)
{ gsub(/''''''/,""); }

# emphasize text in single-quotes 
/'''/ { gsub(/'''('?'?[^'])*'''/, "<strong>&</strong>"); gsub(/'''/,""); }
/''/  { gsub(/''('?[^'])*''/, "<em>&</em>"); gsub(/''/,""); }

#headings
/^-[^-]/ {
    $0 = "<h2>" substr($0, 2) "</h2>"; close_tags(); 
    print next_anchor() $0; next;
}

/^--[^-]/ {
    $0 = "<h3>" substr($0, 3) "</h3>"; close_tags(); 
    print next_anchor() $0; next;
}

/^---[^-]/ { 
    $0 = "<h4>" substr($0, 4) "</h4>"; close_tags(); 
    print next_anchor() $0; next; 
}

# horizontal line
/^----/ { sub(/^----+/, "<hr>"); blankline = 1; close_tags(); print; next; }

/^\t+[*]/ { close_tags("list"); parse_list("ul", "ol"); print; next;}
/^\t+[1]/ { close_tags("list"); parse_list("ol", "ul"); print; next;}
/^\t+[ ]/ { print; next; }

/^ / { 
    close_tags("pre")
    old_ORS = ORS
    ORS = ""
    if (pre != 1) {
        print "<pre>" $0; pre = 1
        blankline = 0
    } else { 
        if (blankline==1) {
            print "\n"; blankline = 0
        }
        print "\n"
        print
    }
    ORS = old_ORS
    next
}

#NR == 1 { print "<p>"; }

{
    close_tags();
        
    # print paragraph when blankline registered
    if (blankline == 1) {
        print "<p>";
        blankline = 0;
    }

    #print line
    print;

}

END {
    $0 = ""
    close_tags();
}

function close_tags(not) {

    # if list is parsed this line print it
    if (not !~ "list") {
        if (list["ol"] > 0)
            parse_list("ol", "ul")
        else if (list["ul"] > 0)
            parse_list("ul", "ol")
    }
    # close monospace
    if (not !~ "pre") {
        if (pre == 1) {
            print "</pre>"
            pre = 0
        }
    }

}

function parse_list(this, other) {

    thislist = list[this]
    otherlist = list[other]
    tabcount = 0

    while(/^\t+[1*]/) {
        sub(/^\t/,"")
        tabcount++
    }
    
    # close foreign tags
    if (otherlist > 0) {
        while(otherlist-- > 0) 
            print "</" other ">"
    }

    # if we are needing more tags we open new
    if (thislist < tabcount) {
        while(thislist++ < tabcount) {
            print "<" this ">"
        }
    }
    # if we are needing less tags we close some
    else if (thislist > tabcount) {
        while(thislist-- != tabcount) {
            print "</" this ">"
        }
    }

    if (tabcount) {
        sub(/^[1*]/,"")
        $0 = "\t<li>" $0
    }
    
    list[other] = 0
    list[this] = tabcount
}

function next_anchor() {
    index_anchor = index_anchor + 1
    return ("<a name=\"anchor_" (index_anchor - 1) "\"></a>")
}

function parse_comment(filename, posttime, edittime, poster) {

    if (edittime > posttime + 60)
        edittime = "<span class=\"tagline1\">&lt;last modified on " \
                   strftime("%D %R", edittime) "&gt;</span>"
    else
        edittime = ""

    if (poster == auth_user)
        extra = "&lt;<a href=\"" scriptname "/" mypage "?post=true&postid=" \
                filename "\">edit</a>&gt;&lt;<a href=\"" scriptname \
                "/" mypage "?postremove=true&postid=" filename "\">remove</a>&gt;"
    else extra = ""

    # anonymous user?
    if (match(poster, /[.]/) > 1) {
        sub("[.]", " from ", poster)
    } else {
        poster = "<a href=\"" scriptname "/" poster "\">" poster "</a>"
    }

    print "<p/><hr><span class=\"tagline\">" " &lt;posted by " poster " on " \
          strftime("%D %R", posttime) "&gt;" extra "</span>" edittime"<p/>"

    system(parser " -v datadir='" datadir \
           "' -v parser='" parser \
           "' -v special_parser='" special_parser \
           "' -v special_list='" special_list \
           "' " datadir "COMMENT/" mypage "/" filename)

    print "<p/>"
}
